# -*- coding: utf-8 -*-
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from assessment.serializers import QuestionSerializer
from assessment.models import Question


@api_view(['GET', ])
def retrieve_question(request, uuid):
    """
    """
    try:
        question = Question.objects.get(pk=uuid)
    except (Question.DoesNotExist, ValueError):
        response_data = {
            "error": {
                "state": "not found",
                "details": "Question object with ID {} could not be found.".format(uuid)
            }
        }
        return Response(response_data, status=status.HTTP_404_NOT_FOUND)

    if question.survey.is_private:
        if request.user.is_authenticated:
            if request.user == question.survey.admin or request.user in question.survey.users.all():
                serializer = QuestionSerializer(question, context={'request': request})
                response_data = serializer.data
                return Response(response_data, status=status.HTTP_200_OK)
            else:
                return Response({"error": "This question is part of a private survey."}, status=status.HTTP_403_FORBIDDEN)
        else:
            return Response({"error": "Please login."}, status=status.HTTP_401_UNAUTHORIZED)
    else:
        serializer = QuestionSerializer(question, context={'request': request})
        response_data = serializer.data
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['POST', ])
def create_question(request):
    """
    create a question for a survey
    """
    if request.user.is_authenticated:
        serializer = QuestionSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    else:
        return Response({"error": "please login"}, status.HTTP_401_UNAUTHORIZED)


@api_view(['PUT', ])
def update_question(request, uuid):
    """
    """
    if request.method == 'PUT':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)


@api_view(['GET', ])
def list_questions(request):
    """
    """
    if request.method == 'GET':
        response_data = {"details": "not implemented"}
        return Response(response_data, status=status.HTTP_200_OK)



# -*- coding: utf-8 -*-
import datetime
import uuid
from django.core.exceptions import ValidationError
from django.db import models
from django.conf import settings
from parler.models import TranslatableModel
from parler.models import TranslatedFields
from django.utils.translation import ugettext as _


class Survey(TranslatableModel):

    _uid = models.UUIDField(
        primary_key=True,
        editable=False,
        default=uuid.uuid4,
    )

    translations = TranslatedFields(
        name=models.CharField(_("name"), max_length=160),
        description=models.TextField(_("description")),
    )

    is_active = models.BooleanField(
        _("active"),
        default=True,
    )

    is_private = models.BooleanField(
        _("private"),
        default=False,
    )

    start_date_time = models.DateTimeField(
        _("start time"),
        auto_now=False,
        default=datetime.datetime.now,
    )

    end_date_time = models.DateTimeField(
        _("end time"),
        auto_now=False,
        null=True,
        blank=True,
    )

    admin = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        null=True,
        related_name="assessment_admin_surveys",
        verbose_name=_('owner'),
    )

    users = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        related_name="assessment_user_surveys",
        blank=True,
    )

    class Meta:
        app_label = 'assessment'
        verbose_name = _("survey")
        verbose_name_plural = _("surveys")

    def __str__(self):
        return self.name


class Question(TranslatableModel):

    TRUEFALSE = 1
    MULTIPLE_CHOICE = 2
    TEXT = 3

    QUESTION_TYPE = (
        (TRUEFALSE, _('true or false')),
        (MULTIPLE_CHOICE, _('multiple choice')),
        (TEXT, _('text')),
    )

    _uid = models.UUIDField(
        primary_key=True,
        editable=False,
        default=uuid.uuid4,
    )

    translations = TranslatedFields(
        question=models.CharField(_("question"), max_length=512),
    )

    survey = models.ForeignKey(
        Survey,
        related_name='questions',
        verbose_name=_("survey")
    )

    is_required = models.BooleanField(
        _("required"),
        default=False,
    )

    of_type = models.IntegerField(
        _("type"),
        choices=QUESTION_TYPE,
        default=TRUEFALSE,
    )

    class Meta:
        app_label = 'assessment'
        verbose_name = _("question")
        verbose_name_plural = _("questions")

    def __str__(self):
        return self.question


class Choice(TranslatableModel):

    _uid = models.UUIDField(
        primary_key=True,
        editable=False,
        default=uuid.uuid4,
    )

    translations = TranslatedFields(
        value=models.CharField(_("value"), max_length=512),
    )

    question = models.ForeignKey(
        Question,
        related_name='choices',
        verbose_name=_("question"),
    )

    is_correct = models.BooleanField(
        _("correct"),
        default=False,
    )

    class Meta:
        app_label = 'assessment'
        verbose_name = _('choice')
        verbose_name_plural = _('choices')

    def __str__(self):
        return self.value


class Result(models.Model):

    _uid = models.UUIDField(
        primary_key=True,
        editable=False,
        default=uuid.uuid4,
    )

    timestamp = models.DateTimeField(
        editable=False,
        default=datetime.datetime.now,
    )

    survey = models.ForeignKey(
        Survey,
        related_name='results',
        verbose_name=_("survey"),
    )

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='results',
        verbose_name=_("user"),
    )

    class Meta:
        app_label = 'assessment'
        verbose_name = _('result')
        verbose_name_plural = _('results')
        unique_together = ('survey', 'user')

    def __str__(self):
        return str(self.pk)

    def clean(self, *args, **kwargs):
        if self.survey.is_private:
            if self.user == self.survey.admin or self.user in self.survey.users.all():
                # The user is allowed to answer this survey.
                # sonarqube complains about how this is structured...
                pass
            else:
                raise ValidationError("User attempted to answer private survey!")


class Answer(models.Model):

    _uid = models.UUIDField(
        primary_key=True,
        editable=False,
        default=uuid.uuid4,
    )

    result = models.ForeignKey(
        Result,
        related_name='answers',
        verbose_name=_("result"),
    )

    question = models.ForeignKey(
        Question,
        related_name='answers',
        verbose_name=_("question"),
    )

    answer = models.TextField(
        _("answer"),
    )

    class Meta:
        app_label = 'assessment'
        unique_together = ('result', 'question')
        verbose_name = _('answer')
        verbose_name_plural = _('answers')

    def __str__(self):
        return self.answer

    def clean(self, *args, **kwargs):
        if self.question not in self.result.survey.questions.all():
            raise ValidationError("User attempted to answer a question not part of this survey!")



"""Urls for the demo of django-assessment"""
from django.conf import settings
from django.conf.urls import url
from django.conf.urls import include

from django.views.static import serve
from django.views.defaults import bad_request
from django.views.defaults import server_error
from django.views.defaults import page_not_found
from django.views.defaults import permission_denied

from rest_framework.authtoken import views as authtoken_views


urlpatterns = [
    url(r'^', include('assessment.urls', namespace='assessment')),
    url(r'^api-token-auth/', authtoken_views.obtain_auth_token),
]

urlpatterns += [
    url(r'^400/$', bad_request),
    url(r'^403/$', permission_denied),
    url(r'^404/$', page_not_found),
    url(r'^500/$', server_error),
]

if settings.DEBUG:
    urlpatterns += [
        url(r'^media/(?P<path>.*)$', serve,
            {'document_root': settings.MEDIA_ROOT})
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-31 05:30
from __future__ import unicode_literals

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import parler.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Answer',
            fields=[
                ('_uid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('answer', models.TextField(verbose_name='answer')),
            ],
            options={
                'verbose_name_plural': 'answers',
                'verbose_name': 'answer',
            },
        ),
        migrations.CreateModel(
            name='Choice',
            fields=[
                ('_uid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('is_correct', models.BooleanField(default=False, verbose_name='correct')),
            ],
            options={
                'verbose_name_plural': 'choices',
                'verbose_name': 'choice',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ChoiceTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('value', models.CharField(max_length=512, verbose_name='value')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='assessment.Choice')),
            ],
            options={
                'verbose_name': 'choice Translation',
                'db_tablespace': '',
                'default_permissions': (),
                'db_table': 'assessment_choice_translation',
                'managed': True,
            },
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('_uid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('is_required', models.BooleanField(default=False, verbose_name='required')),
                ('of_type', models.IntegerField(choices=[(1, 'true or false'), (2, 'multiple choice'), (3, 'text')], default=1, verbose_name='type')),
            ],
            options={
                'verbose_name_plural': 'questions',
                'verbose_name': 'question',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='QuestionTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('question', models.CharField(max_length=512, verbose_name='question')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='assessment.Question')),
            ],
            options={
                'verbose_name': 'question Translation',
                'db_tablespace': '',
                'default_permissions': (),
                'db_table': 'assessment_question_translation',
                'managed': True,
            },
        ),
        migrations.CreateModel(
            name='Result',
            fields=[
                ('_uid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('timestamp', models.DateTimeField(default=datetime.datetime.now, editable=False)),
            ],
            options={
                'verbose_name_plural': 'results',
                'verbose_name': 'result',
            },
        ),
        migrations.CreateModel(
            name='Survey',
            fields=[
                ('_uid', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('is_active', models.BooleanField(default=True, verbose_name='active')),
                ('is_private', models.BooleanField(default=False, verbose_name='private')),
                ('start_date_time', models.DateTimeField(default=datetime.datetime.now, verbose_name='start time')),
                ('end_date_time', models.DateTimeField(blank=True, null=True, verbose_name='end time')),
                ('admin', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='assessment_admin_surveys', to=settings.AUTH_USER_MODEL, verbose_name='owner')),
                ('users', models.ManyToManyField(blank=True, related_name='assessment_user_surveys', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'surveys',
                'verbose_name': 'survey',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SurveyTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=160, verbose_name='name')),
                ('slug', models.SlugField(max_length=160, unique=True, verbose_name='slug')),
                ('description', models.TextField(verbose_name='description')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='assessment.Survey')),
            ],
            options={
                'verbose_name': 'survey Translation',
                'db_tablespace': '',
                'default_permissions': (),
                'db_table': 'assessment_survey_translation',
                'managed': True,
            },
        ),
        migrations.AddField(
            model_name='result',
            name='survey',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='results', to='assessment.Survey', verbose_name='survey'),
        ),
        migrations.AddField(
            model_name='result',
            name='user',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='results', to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
        migrations.AddField(
            model_name='question',
            name='survey',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='assessment.Survey', verbose_name='survey'),
        ),
        migrations.AddField(
            model_name='choice',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='choices', to='assessment.Question', verbose_name='question'),
        ),
        migrations.AddField(
            model_name='answer',
            name='question',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='assessment.Question', verbose_name='question'),
        ),
        migrations.AddField(
            model_name='answer',
            name='result',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='assessment.Result', verbose_name='result'),
        ),
        migrations.AlterUniqueTogether(
            name='surveytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='result',
            unique_together=set([('survey', 'user')]),
        ),
        migrations.AlterUniqueTogether(
            name='questiontranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='choicetranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='answer',
            unique_together=set([('result', 'question')]),
        ),
    ]



# -*- coding: utf-8 -*-
from django.conf.urls import url
from assessment.views.api import answer


urlpatterns = [
    url(
        regex=r'^$',
        view=answer.list_answers,
        name='list_answers',
    ),
    url(
        regex=r'^create/$',
        view=answer.create_answer,
        name='create_answer',
    ),
    url(
        regex=r'^update/(?P<uuid>[-\w]+)/$',
        view=answer.update_answer,
        name='update_answer',
    ),
    url(
        regex=r'^(?P<uuid>[-\w]+)/$',
        view=answer.retrieve_answer,
        name='retrieve_answer',
    ),
]



"""Setup script of django-assessment"""
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
from setuptools import find_packages

import assessment

setup(
    name="django-assessment",
    version=assessment.__version__,

    description="Django Assessment package",
    long_description="Django Assessment package",
    keywords="django, assessment, quiz",

    author=assessment.__author__,
    author_email=assessment.__email__,
    url=assessment.__url__,

    packages=find_packages(exclude=("docs*", "tests*")),
    classifiers=[
        "Framework :: Django",
        "Development Status :: 1 - Planning",
        "Environment :: Web Environment",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    license=assessment.__license__,
    include_package_data=True,
    zip_safe=False,
    install_requires = [
        "Django>=1.8",
        "djangorestframework>=3.4",
        "Markdown>=2.6",
        "django-filter>=1.0",
        "django_parler>=1.8",
        "django-parler-rest>=1.4",
        "django-cors-headers>=2.0",
    ],
)



__version__ = '0.3.0'

__author__ = 'Joost VanDorp'

__email__ = 'joostvandorp@gmail.com'

__url__ = 'https://github.com/vandorjw/django-assessment'

__license__ = 'MIT'

VERSION = __version__  # synonym
